w,h = 10,10


class Cell():
  def __init__(self,x,y,w):
    self.x = x
    self.y = y
    self.f = 0
    self.g = 0
    self.h = 0
    self.neighbors = []
    self.previous = None
    self.wall = w
  
  def show(self,color):
    if self.wall:
      color = (0,0,0)
    pixel(self.x*5,self.y*5,color,5)
  
  def addNeighbors(self,grid):
    if self.x > 0:
      self.neighbors.append(grid[self.y][self.x-1])
    if self.y > 0:
      self.neighbors.append(grid[self.y-1][self.x])
    if self.x < w-1:
      self.neighbors.append(grid[self.y][self.x+1])
    if self.y < h-1:
      self.neighbors.append(grid[self.y+1][self.x])


def getDir(a,b):
  if a == None or b == None:
    return -1
  
  if a.y > b.y:
    return 0
  elif a.x < b.x:
    return 1
  elif a.y < b.y:
    return 2
  elif a.x > b.x:
    return 3
  else:
    return -1

def heuristic(a,b):
  return abs(a.x-b.x)+abs(a.y-b.y)

def astar(s,e,walls):
  grid = [[Cell(x,y,True) if (x,y) in walls else Cell(x,y,False) for x in range(w)] for y in range(h)]
  for y in range(h):
    for x in range(w):
      grid[y][x].addNeighbors(grid)
  
  openSet = []
  closedSet = []
  start = grid[s[1]][s[0]]
  end = grid[e[1]][e[0]]
  
  openSet.append(start)
  
  done = False
  while not done:
    if len(openSet) > 0:
      current = openSet[0]
      for cell in openSet:
        if cell.f < current.f:
          current = cell
      
      if current == end:
        done = True
      
      openSet.remove(current)
      closedSet.append(current)
      
      for neighbor in current.neighbors:
        if neighbor in closedSet or neighbor.wall:
          continue
        
        tempG = current.g+1
        if neighbor not in openSet:
          openSet.append(neighbor)
        elif tempG >= neighbor.g:
          continue
        
        neighbor.h = heuristic(neighbor,end)
        neighbor.g = tempG
        neighbor.f = neighbor.g+neighbor.h
        neighbor.previous = current
    else:
      done = True
    
    path = []
    temp = current
    path.append(getDir(temp.previous,temp))
    while temp.previous:
      path.append(getDir(temp.previous,temp))
      temp = temp.previous
    path.reverse()
  
  return path


#print(astar((3,5),(w-1,h-1),[(2,5),(1,5),(0,5)]))
