from astar import *
from formes import *
from random import randint


size = 5


class Food():
  def __init__(self):
    self.x = randint(0,w-1)
    self.y = randint(0,h-1)
  
  def updatePos(self):
    good = False
    while not good:
      self.x = randint(0,w-1)
      self.y = randint(0,h-1)
      good = True
      for part in [snake.h]+snake.body:
        if self.x == part.x and self.y == part.y:
          good = False
  
  def show(self):
    pixel(self.x*size,self.y*size,(255,0,0),size)

class Part():
  def __init__(self,x,y):
    self.x = x
    self.y = y
  
  def show(self,c):
    pixel(self.x*size,self.y*size,c,size)

class Snake():
  def __init__(self,length):
    self.h = Part(w//3,h//2)
    self.body = [Part(w//3-(i+1),h//2) for i in range(length-1)]
    self.dx = 1
    self.dy = 0
    self.dead = False
  
  def turn(self,d):
    if d == 0:
      self.dx = 0
      self.dy = -1
    elif d == 1:
      self.dx = 1
      self.dy = 0
    elif d == 2:
      self.dx = 0
      self.dy = 1
    elif d == 3:
      self.dx = -1
      self.dy = 0
  
  def move(self):
    if self.h.x+self.dx < 0:
      self.dead = True
      return
    elif self.h.x+self.dx > w-1:
      self.dead = True
      return
    elif self.h.y+self.dy < 0:
      self.dead = True
      return
    elif self.h.y+self.dy > h-1:
      self.dead = True
      return
    
    for part in self.body:
      if self.h.x+self.dx == part.x and self.h.y+self.dy == part.y:
        self.dead = True
        return
    
    self.body.insert(0,Part(self.h.x,self.h.y))
    self.h.x += self.dx
    self.h.y += self.dy
    if not (self.h.x == food.x and self.h.y == food.y):
      self.body.pop(len(self.body)-1)
  
  def show(self):
    self.h.show((0,0,255))
    for part in self.body:
      part.show((0,0,180))


snake = Snake(3)
food = Food()
score = 0

while not snake.dead:
  snake.move()
  if snake.h.x == food.x and snake.h.y == food.y:
    food.updatePos()
    score += 1
  walls = [(part.x,part.y) for part in snake.body]
  path = astar((snake.h.x,snake.h.y),(food.x,food.y),walls)
  snake.turn(path[0])
  
  for y in range(h):
    for x in range(w):
      pixel(x*size,y*size,(0,255,0),size)
  snake.show()
  food.show()
  show_screen()

#print("GG your'e dead !")
draw_string(0,h*size+size,"GG your'e dead ! Score : "+str(score),(255,0,0),"medium")
